module ExpressAdmin
  module Components
    class FileUpload < ExpressTemplates::Components::Forms::FormComponent

      has_option :action, 'The form action containing the resource path or url.'
      has_option :max_file_size, 'The maximum file size a user can upload', type: :int

      contains -> {
        div(class: 'dropzone-previews') {
          div(class: 'dz-default dz-message') {
            span(class: 'dz-message-instruction') {'Drop files here or click to upload.'}
          }
        }

        div(class: 'dz-preview dz-file-preview', id: 'preview-template'){
          div(class: 'dz-error-message hide') {
            span('data-dz-errormessage' => 'true')
          }
          div(class: 'dz-image'){
            img('data-dz-thumbnail' => true)
          }
          div(class: 'dz-close hide') {
            icon_link('close-round', 'data-dz-remove' => true)
          }
          div(class: 'dz-details hide'){
            div(class: 'dz-filename'){
              span('data-dz-name' => true)
            }
            div(class: 'dz-size', 'data-dz-size' => true)
          }
          div(class: 'dz-progress') {
            span(class: 'dz-upload', 'data-dz-uploadprogress' => true)
          }
          div(class: 'dz-success-mark hide') {
            icon('checkmark-circled')
          }
          div(class: 'dz-error-mark hide') {
            icon('close-circled')
          }
        }

        content_for(:page_javascript) {
          script {
            %Q(
              $(function() {
                previewNode = $('#preview-template').get(0).outerHTML;
                $('#preview-template').remove();
                var submitButton;
                Dropzone.options.#{config[:id].to_s.camelize(:lower)} = {
                  url: window.location.origin + '#{config[:action]}',
                  paramName: '#{config[:id]}[file]',
                  autoProcessQueue: false,
                  uploadMultiple: false,
                  maxFiles: 1,
                  maxFilesize: #{max_file_size},
                  previewTemplate: previewNode,
                  previewsContainer: '.dropzone-previews',
                  clickable: '.dropzone-previews',
                  init: function() {
                    var myDropzone = this;
                    submitButton = this.element.querySelector('input[type=submit]');

                    $(submitButton).on('click', function(e) {
                      e.preventDefault();
                      myDropzone.processQueue();
                    });

                    this.on('addedfile', function(file) {
                      if (this.files[1]) {
                        this.removeFile(this.files[0]);
                      }
                      $('.dz-message.dz-default').addClass('hide');
                    });

                    this.on('removedfile', function() {
                      $('.dz-message.dz-default').removeClass('hide');
                      $(submitButton).removeClass('disabled');
                    });

                    this.on('processing', function() {
                      $('.dz-image').zIndex('-999');
                      $('.dz-image > img').addClass('blur');
                      $(submitButton).addClass('disabled');
                    });

                    this.on('maxfilesexceeded', function(file) {
                      this.removeAllFiles();
                      this.addFile(file);
                    });

                    this.on('success', function(file) {
                      $('.dz-progress').css('opacity', 0);
                      $('.dz-success-mark').removeClass('hide');
                      setTimeout( function() {
                        $('.dz-image').zIndex('999');
                        $('.dz-image > img').removeClass('blur');
                        location.reload();
                      }, 2000);
                    });

                    this.on('complete', function(file) {
                      if (file.accepted === false) {
                        $('.dz-progress').css('opacity', 0);
                        setTimeout(function() { $('.dz-error-mark').removeClass('hide') }, 500);

                        setTimeout(function() {
                          $('.dz-image').zIndex('999');
                          $('.dz-image > img').removeClass('blur');
                          $('.dz-error-message').removeClass('hide');
                          $(submitButton).addClass('disabled');
                        }, 3000);
                      }
                    });

                    this.on('drop', function() {
                      $('.dz-message.dz-default').addClass('hide');
                    });
                  },
                }
              });
            ).html_safe
          }
        }
      }

      def max_file_size
        config[:max_file_size]
      end
    end
  end
end
